from typing import Any, Callable, Dict, Optional, Tuple, Type, TypeVar, Union
from ..model.error import (
    FieldValidationIssue,
    ModelHTTPError,
    ModelValidationError,
)
from functools import wraps
from pydantic import BaseModel, ValidationError as PydanticValidationError
import inspect

__all__ = ["validate_request_response"]

T = TypeVar("T", bound=BaseModel)


class ValidationHelper:
    """验证逻辑公共工具类"""

    @staticmethod
    def safe_validate(
        model: Type[T], input_data: Union[Any]
    ) -> Tuple[Optional[T], Optional[ModelValidationError]]:
        """
        泛型安全校验函数

        Args:
            model: 继承自BaseModel的泛型模型类
            input_data: 待校验的原始数据字典

        Returns:
            - Tuple[validated_model, None] 验证成功
            - Tuple[None, ModelValidationError] 验证失败

        Example:
            >>> class UserModel(BaseModel):
            ...     name: str
            >>> data, error = safe_validate(UserModel, {"name": "Alice"})
            >>> print(isinstance(data, UserModel))  # True
        """
        try:
            validated = model.model_validate(input_data)
            return validated, None

        except PydanticValidationError as ex:
            issues = []
            for err in ex.errors():
                issues.append(
                    FieldValidationIssue(
                        field=".".join(map(str, err["loc"])),
                        message=err["msg"],
                        error_type=err["type"],
                    )
                )

            return None, ModelValidationError(
                detail=f"Invalid {model.__name__} payload",
                issues=issues,
            )

    @staticmethod
    def prepare_input_data(
        func: Callable, *args, **kwargs
    ) -> Tuple[Optional[Dict], Optional[ModelValidationError]]:
        """准备待验证的输入数据"""
        # 获取被装饰函数的参数名称
        sig = inspect.signature(func)
        try:
            bound_args = sig.bind(*args, **kwargs)
        except TypeError as e:
            return None, ModelValidationError(str(e))
        return {**bound_args.arguments}, None

    @staticmethod
    def validate_request(
        request_model: Type[T], input_data: dict, func: Callable
    ) -> Tuple[Optional[Dict], Optional[ModelValidationError]]:
        """请求参数验证（同步）"""
        is_instance_method = (
            len(input_data) > 0
            and hasattr(list(input_data.values())[0], func.__name__)
            and inspect.ismethod(getattr(list(input_data.values())[0], func.__name__))
        )

        if is_instance_method:
            instance = list(input_data.values())[0]
            validation_data = {
                k: v for k, v in input_data.items() if k != list(input_data.keys())[0]
            }
        else:
            validation_data = input_data

        req, err = ValidationHelper.safe_validate(request_model, validation_data)
        if err is not None:
            return None, err

        validated_data = req.model_dump(exclude_unset=True)
        if is_instance_method:
            validated_data.update({list(input_data.keys())[0]: instance})

        return validated_data, None

    @staticmethod
    def validate_response(
        response_model: Type[T], result: Any
    ) -> Tuple[Optional[Dict], Optional[ModelValidationError]]:
        """响应数据验证（同步）"""
        resp, err = ValidationHelper.safe_validate(response_model, result)
        if err is not None:
            return None, err

        return resp, None


def validate_request_response(
    request_model: Optional[type[T]] = None,
    response_model: Optional[type[T]] = None,
) -> Callable:
    """
    同步装饰器工厂，用于验证请求参数和响应数据

    Args:
        request_model: 请求参数的Pydantic模型
        response_model: 响应数据的Pydantic模型
    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def wrapper(*args, **kwargs) -> Union[T, ModelHTTPError, ModelValidationError]:
            # 构建输入数据（保留self/cls引用）
            input_data, err = ValidationHelper.prepare_input_data(func, *args, **kwargs)
            if err is not None:
                return err

            if request_model is not None:
                input_data, err = ValidationHelper.validate_request(
                    request_model, input_data, func
                )
                if err is not None:
                    return err

            result = func(**input_data)
            if isinstance(result, ModelHTTPError):
                return result

            # 验证响应数据
            if response_model is not None:
                result, err = ValidationHelper.validate_response(response_model, result)
                if err is not None:
                    return err

                return result

            return result

        return wrapper

    return decorator


def async_validate_request_response(
    request_model: Optional[type[T]] = None,
    response_model: Optional[type[T]] = None,
) -> Callable:
    """
    异步装饰器工厂，用于验证请求参数和响应数据

    Args:
        request_model: 请求参数的Pydantic模型
        response_model: 响应数据的Pydantic模型
    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        async def async_wrapper(
            *args, **kwargs
        ) -> Union[T, ModelHTTPError, ModelValidationError]:
            # 构建输入数据（保留self/cls引用）
            input_data, err = ValidationHelper.prepare_input_data(func, *args, **kwargs)
            if err is not None:
                return err

            if request_model is not None:
                input_data, err = ValidationHelper.validate_request(
                    request_model, input_data, func
                )
                if err is not None:
                    return err

            result = await func(**input_data)
            if isinstance(result, ModelHTTPError):
                return result

            # 验证响应数据
            if response_model is not None:
                result, err = ValidationHelper.validate_response(response_model, result)
                if err is not None:
                    return err

                return result

            return result

        return async_wrapper

    return decorator



from typing import List, Optional
from .validation import async_validate_request_response
from .stream import Stream
from ..client.http_client import HTTPClient
from ..model.task import (
    QueueItem,
    StreamAllReq,
    StreamBatchReq,
    StreamSingleReq,
    StreamResult,
)
from uuid import UUID
import asyncio

__all__ = ["Task"]


class Task:
    URL_PREFIX = "/tasks"

    URL_ENDPOINTS = {
        "get_detail": "/{task_id}",
        "get_stream": "/{task_id}/stream",
    }

    def __init__(self, client: HTTPClient, api_version: str, max_connections: int = 10):
        self.client = client
        self.stream = Stream(client.base_url, max_connections)
        self.url_prefix = f"{api_version}{self.URL_PREFIX}"

    @async_validate_request_response(StreamSingleReq, QueueItem)
    async def get_stream_single(
        self,
        task_id: UUID,
        trace_id: UUID,
        client_timeout: float,
        wait_timeout: Optional[float] = None,
    ) -> QueueItem:
        url = f"{self.url_prefix}{self.URL_ENDPOINTS['get_stream'].format(task_id=task_id)}"

        try:
            report = await asyncio.wait_for(
                self.stream.start_single_stream(trace_id, url, client_timeout),
                wait_timeout,
            )
            return report
        finally:
            await self.stream.cleanup()

    @async_validate_request_response(StreamBatchReq)
    async def get_stream_batch(
        self, task_ids: List[UUID], trace_ids: List[UUID], client_timeout: float
    ) -> asyncio.Queue[QueueItem]:
        urls = [
            f"{self.url_prefix}{self.URL_ENDPOINTS['get_stream'].format(task_id=task_id)}"
            for task_id in task_ids
        ]

        queue = await self.stream.add_result_queue()
        await self.stream.start_multiple_stream(trace_ids, urls, client_timeout)
        return queue

    @async_validate_request_response(StreamAllReq, StreamResult)
    async def get_stream_all(
        self,
        task_ids: List[UUID],
        trace_ids: List[UUID],
        client_timeout: float,
        wait_timeout: Optional[float] = None,
    ) -> StreamResult:
        urls = [
            f"{self.url_prefix}{self.URL_ENDPOINTS['get_stream'].format(task_id=task_id)}"
            for task_id in task_ids
        ]

        await self.stream.start_multiple_stream(trace_ids, urls, client_timeout)
        report = await self.stream.client_manager.wait_all(wait_timeout)
        await self.stream.cleanup()

        return report



from typing import Any, Dict, List, Optional
from rcabench.logger import logger
from rcabench.rcabench import RCABenchSDK
import asyncio

BASE_URL = "http://127.0.0.1:8082"
sdk = RCABenchSDK(BASE_URL)


async def test_injection_and_building_dataset_batch(
    benchmark: str,
    interval: int,
    pre_duration: int,
    specs: List[Dict[str, Any]],
    num_consumers: int,
    max_items_per_consumer: int,
    per_consumer_timeout: float,
):
    assert num_consumers <= len(specs)
    sdk = RCABenchSDK(BASE_URL)

    resp = sdk.injection.submit(benchmark, interval, pre_duration, specs)
    logger.info(resp)

    traces = resp.traces

    task_ids = [trace.head_task_id for trace in traces]
    trace_ids = [trace.trace_id for trace in traces]
    queue = await sdk.task.get_stream_batch(task_ids, trace_ids, interval * 60)

    results = await run_consumers(
        queue,
        num_consumers,
        max_items_per_consumer,
        per_consumer_timeout,
    )

    await sdk.task.stream.cleanup()
    return results


async def run_consumers(
    queue: asyncio.Queue,
    num_consumers: int,
    max_items_per_consumer: int,
    per_consumer_timeout: Optional[float] = None,
):
    all_results = {}
    tasks = [
        asyncio.create_task(
            consumer_task(
                i, queue, max_items_per_consumer, per_consumer_timeout * (i + 1)
            )
        )
        for i in range(num_consumers)
    ]

    results = await asyncio.gather(*tasks, return_exceptions=True)

    for i, result in enumerate(results):
        if isinstance(result, Exception):
            logger.error(f"Consumer {i} failed: {str(result)}")
            all_results[f"Batch-{i}"] = None
        else:
            all_results[f"Batch-{i}"] = result

    return all_results


async def consumer_task(
    consumer_id: int,
    queue: asyncio.Queue,
    max_num: int,
    timeout: Optional[float] = None,
):
    try:
        results = await consumer(queue, max_num, timeout)
        logger.info(f"Consumer-{consumer_id} completed")
        return results
    except asyncio.TimeoutError:
        logger.error(f"Consumer-{consumer_id} timed out")
        raise
    except Exception as e:
        logger.error(f"Consumer-{consumer_id} error: {str(e)}")
        raise


async def consumer(queue: asyncio.Queue, max_num: int, timeout: Optional[float] = None):
    results = []
    count = 0
    while count < max_num:
        try:
            item = await asyncio.wait_for(queue.get(), timeout)
            results.append(item)
            count += 1
        except asyncio.TimeoutError:
            logger.warning("Timeout while waiting for queue item")
            raise

    return results


async def main(total_timeout: Optional[float] = None):
    payload = [
        "clickhouse",
        3,
        1,
        [
            {
                "children": {
                    "1": {
                        "children": {
                            "0": {"value": 1},
                            "1": {"value": 0},
                            "2": {"value": 42},
                        }
                    },
                },
                "value": 1,
            },
        ],
        1,
        1,
        3 * 60,
    ]

    results = {}
    try:
        results = await asyncio.wait_for(
            test_injection_and_building_dataset_batch(*payload), timeout=total_timeout
        )
        logger.info(f"Final results: {results}")
    except asyncio.TimeoutError:
        logger.error("Total time out")
    finally:
        logger.info(f"Final results: {results}")


if __name__ == "__main__":
    asyncio.run(main())
